def get_status(verbose: bool = False):
    # https://pytorch.org/docs/stable/generated/torch.cuda.is_available.html
    try:
        import torch

        if torch.cuda.is_available():
            return True
    except Exception as e:
        if verbose:
            print(e)

    return False


# https://stackoverflow.com/questions/60987997/why-torch-cuda-is-available-returns-false-even-after-installing-pytorch-with
def validate():
    try:
        import torch

        print(torch.zeros(1).cuda())
        return True
    except Exception as e:
        print(e)
        return False



from abcli.plugins import testing
import pytest
from abcli import file
from abcli.modules import objects
from abcli import env
from abcli.plugins.testing import download_object


@pytest.mark.parametrize(
    ["object_name"],
    [
        [env.VANWATCH_TEST_OBJECT],
    ],
)
def test_download_object(object_name):
    assert download_object(object_name)

    list_of_files = [
        file.name_and_extension(filename)
        for filename in objects.list_of_files(object_name=object_name)
    ]

    assert "vancouver.json" in list_of_files, objects.path_of(
        object_name=object_name,
        filename="vancouver.json",
    )

    assert file.exist(
        objects.path_of(
            object_name=object_name,
            filename="vancouver.json",
        )
    )



import os
from abcli import fullname
from abcli import env, file
from abcli import string
from . import HOST_NAME, HOST_TAGS, NAME
from abcli.logger import logger, crash_report


def get_name(cache=True):
    global HOST_NAME

    if cache and HOST_NAME is not None:
        return HOST_NAME

    HOST_NAME = get_name_()
    return HOST_NAME


def get_name_():
    default = string.random_(5)

    if is_docker():
        return os.getenv("abcli_container_id", default)

    if is_ec2():
        return os.getenv("abcli_ec2_instance_id", default)

    if is_jetson():
        return os.getenv("abcli_jetson_nano_serial_number", default)

    if is_ubuntu():
        return os.getenv("abcli_ubuntu_computer_id", default)

    if is_mac():
        return os.getenv("USER", default)

    if is_jupyter():
        return "Jupyter-Notebook"

    try:
        if is_rpi():
            # https://www.raspberrypi-spy.co.uk/2012/09/getting-your-raspberry-pi-serial-number-using-python/
            with open("/proc/cpuinfo", "r") as fp:
                for line in fp:
                    if line.startswith("Serial"):
                        return line[10:26]

        with open("/sys/firmware/devicetree/base/serial-number", "r") as fp:
            for line in fp:
                return line.strip().replace(chr(0), "")

    except:
        crash_report(f"-{NAME}: get_name(): failed.")

    return default


def get_seed_filename():
    return (
        "/media/abcli/SEED/abcli/jetson.sh"
        if is_jetson()
        else (
            "/Volumes/seed/abcli/ubuntu.sh"
            if is_ubuntu()
            else "/media/pi/SEED/abcli/rpi.sh" if is_rpi() else ""
        )
    )


def get_tags(cache=True):
    from abcli.plugins import tags

    global HOST_TAGS

    if cache and HOST_TAGS is not None:
        return HOST_TAGS

    HOST_TAGS = tags.get(get_name(cache=cache))
    return HOST_TAGS


def is_aws_batch():
    return os.getenv("abcli_is_aws_batch", "false") == "true"


def is_docker():
    return os.getenv("abcli_is_docker", "false") == "true"


def is_ec2():
    return os.getenv("abcli_is_ec2", "false") == "true"


def is_github_workflow():
    return os.getenv("abcli_is_github_workflow", "false") == "true"


def is_headless():
    return os.getenv("abcli_is_headless", "false") == "true"


def is_jetson():
    return os.getenv("abcli_is_jetson", "false") == "true"


# https://github.com/ultralytics/yolov5/blob/master/utils/general.py#LL79C18-L79C18
def is_jupyter():
    """
    Check if the current script is running inside a Jupyter Notebook.
    Verified on Colab, Jupyterlab, Kaggle, Paperspace.
    Returns:
        bool: True if running inside a Jupyter Notebook, False otherwise.
    """
    try:
        from IPython import get_ipython

        return get_ipython() is not None
    except:
        return False


def is_mac():
    return os.getenv("abcli_is_mac", "false") == "true"


def is_rpi():
    return os.getenv("abcli_is_rpi", "false") == "true"


def is_ubuntu():
    return os.getenv("abcli_is_ubuntu", "false") == "true"


def shell(
    command,
    clean_after=False,
    return_output=False,
    work_dir=".",
):
    """execute command in shell.

    Args:
        command (str): command
        keep_blank (bool, optional): keep blank lines in output. Defaults to True.
        clean_after (bool, optional): delete output file. Defaults to False.
        return_output (bool, optional): return output. Defaults to False.
        work_dir (str, optional): working directory. Defaults to ".".
        strip (bool, optional): strip output. Defaults to False.

    Returns:
        bool: success.
        (List[str], optional): output.
    """
    logger.debug(f"host.shell({command})")

    success = True
    output = []

    if return_output:
        output_filename = file.auxiliary("shell", "txt")
        command += f" > {output_filename}"

    current_path = os.getcwd()
    try:
        os.chdir(work_dir)

        try:
            os.system(command)
        except:
            crash_report(f"host.shell({command}) failed")
            success = False

    finally:
        os.chdir(current_path)

    if success and return_output:
        success, output = file.load_text(output_filename)

        if clean_after:
            file.delete(output_filename)

    return (success, output) if return_output else success


def signature():
    import platform

    return (
        [fullname()]
        + list(
            set(
                [
                    env.abcli_hostname,
                    get_name(),
                ]
            )
        )
        + tensor_processing_signature()
        + [
            "Python {}".format(platform.python_version()),
            "{} {}".format(platform.system(), platform.release()),
        ]
        + ([env.abcli_wifi_ssid] if env.abcli_wifi_ssid else [])
    )


def tensor_processing_signature():
    output = []

    try:
        import tensorflow

        output += [f"TensorFlow {tensorflow.__version__}"]
    except:
        pass

    try:
        from tensorflow import keras

        output += [f"Keras {keras.__version__}"]
    except:
        pass

    try:
        import tflite_runtime.interpreter as tflite

        output += ["TensorFlow Lite"]
    except:
        pass

    try:
        import torch

        output += [f"torch-{torch.__version__}"]
    except:
        pass

    return output



import pytest
from abcli import env
from abcli.plugins.functions import list_of_external, get_plugin_name, get_module_name


@pytest.mark.skipif(
    env.abcli_is_github_workflow == "true",
    reason="plugins are not present in the github workflow.",
)
def test_list_of_external():
    assert list_of_external()


@pytest.mark.skipif(
    env.abcli_is_github_workflow == "true",
    reason="plugins are not present in the github workflow.",
)
@pytest.mark.parametrize(
    [
        "repo_name",
        "plugin_name",
        "module_name",
    ],
    [
        [
            "awesome-bash-cli",
            "abcli",
            "abcli",
        ],
        [
            "CV",
            "CV",
            "abadpour",
        ],
        [
            "roofAI",
            "roofAI",
            "roofAI",
        ],
        [
            "vancouver-watching",
            "vancouver_watching",
            "vancouver_watching",
        ],
        [
            "giza",
            "giza",
            "gizai",
        ],
        [
            "hubble",
            "hubble",
            "hubblescope",
        ],
        [
            "aiart",
            "aiart",
            "articraft",
        ],
    ],
)
def test_get(
    repo_name: str,
    plugin_name: str,
    module_name: str,
):
    assert get_plugin_name(repo_name) == plugin_name
    assert get_module_name(repo_name) == module_name



import os
import pathlib
import shutil
from abcli import env
from abcli.path import NAME
from abcli import string
from abcli.logger import logger, crash_report


def absolute(path, reference=None):
    """return absolute path.

    Args:
        path (str): path.
        reference (str, optional): reference. Defaults to None=current path.

    Returns:
        str: absolute path.
    """
    if reference is None:
        reference = current()

    path = path.replace("/", os.sep)
    reference = reference.replace("/", os.sep)

    return (
        reference
        if not path
        else (
            path
            if path[0] != "."
            else str(pathlib.Path(os.path.join(reference, path)).resolve())
        )
    )


def auxiliary(nickname, add_timestamp=True):
    """generate auxiliary path.

    Args:
        nickname (str): nickname
        add_timestamp (bool, optional): add timestamp. Defaults to True.

    Returns:
        str: auxiliary path.
    """
    path = os.path.join(
        env.abcli_object_path,
        "auxiliary",
        "-".join(
            [nickname]
            + (
                [
                    string.pretty_date(
                        as_filename=True,
                        squeeze=True,
                        unique=True,
                    )
                ]
                if add_timestamp
                else []
            )
        ),
    )

    create(path)

    return path


def copy(source, destination):
    """copy source to destination.

    Args:
        source (str): source.
        destination (str): destination.

    Returns:
        bool: success.
    """
    if not create(parent(destination)):
        return False

    try:
        shutil.copytree(source, destination)
        return True
    except:
        crash_report(f"-{NAME}: copy({source},{destination}): failed.")
        return False


def create(path, log=False):
    """create path.

    Args:
        path (path): path.
        log (bool, optional): log. Defaults to False.

    Returns:
        bool: success.
    """
    if not path or exist(path):
        return True

    try:
        os.makedirs(path)
    except:
        crash_report(f"-{NAME}: create({path}): failed.")
        return False

    if log:
        logger.info(f"{NAME}.create({path})")

    return True


def current():
    """return current path.

    Returns:
        str: current path.
    """
    return os.getcwd()


def delete(path):
    """delete path.

    Args:
        path (str): path.

    Returns:
        bool: success.
    """
    try:
        # https://docs.python.org/3/library/shutil.html#shutil.rmtree
        shutil.rmtree(path)
        return True
    except:
        crash_report(f"-{NAME}: delete({path}): failed.")
        return False


def exist(path):
    """does path exist?

    Args:
        path (str): path.

    Returns:
        bool: does path exist?
    """
    return os.path.exists(path) and os.path.isdir(path)


def list_of(path, recursive=False):
    """return list of folders in path.

    Args:
        path (_type_): path.
        recursive (bool, optional): look in all sub-folders as well. Defaults to False.

    Returns:
        list: list of paths.
    """
    if not exist(path):
        return []

    # http://stackabuse.com/python-list-files-in-a-directory/
    output = []
    try:
        for entry in os.scandir(path):
            if entry.is_file():
                continue

            path_name = os.path.join(path, entry.name)

            output.append(path_name)

            if recursive:
                output += list_of(path_name, recursive=recursive)
    except:
        crash_report(f"-{NAME}: list_of({path}): failed.")

    return output


def move(source, destination):
    """move source to destination.

    Args:
        source (str): source.
        destination (str): destination.

    Returns:
        bool: success.
    """
    if not create(parent(destination)):
        return False

    try:
        shutil.move(source, destination)
        return True
    except:
        crash_report(f"-{NAME}: move({source},{destination}): failed.")
        return False


def name(path):
    """return name of path.

    Args:
        path (str): path.

    Returns:
        str: path name.
    """
    if not path:
        return path

    if path[-1] == os.sep:
        path = path[:-1]

    path_components = path.split(os.sep)
    return "" if not path_components else path_components[-1]


def parent(path, depth=1):
    """return parent for path.

    Args:
        path (str): path.
        depth (int, optional): parent index. Defaults to 1.

    Returns:
        str: parent name.
    """
    # Add os.sep at the end, if it already does not exist.
    if path != "":
        if path[-1] != os.sep:
            path = path + os.sep

    return os.sep.join(path.split(os.sep)[: -depth - 1]) + os.sep


def relative(path, reference=None):
    """return relative path.

    Args:
        path (str): path
        reference (_type_, optional): reference path. Defaults to None.

    Returns:
        str: relative path.
    """
    # https://stackoverflow.com/a/918174
    return os.path.relpath(path, current() if reference is None else reference)



import argparse
from abcli import VERSION
from abcli.plugins import NAME
from abcli.plugins.functions import get_module_name, get_plugin_name, list_of_external
from abcli.logger import logger
from blueness.argparse.generic import sys_exit


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help="get_module_name|get_plugin_name|list_of_external",
)
parser.add_argument(
    "--delim",
    type=str,
    default=", ",
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--repo_names",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--repo_name",
    default="",
    type=str,
)
parser.add_argument(
    "--plugin_name",
    default="",
    type=str,
)
parser.add_argument(
    "--from_plugin_name",
    default=0,
    type=int,
    help="0|1",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "get_module_name":
    success = True
    print(get_module_name(args.repo_name))
elif args.task == "get_plugin_name":
    success = True
    print(get_plugin_name(args.repo_name))
elif args.task == "list_of_external":
    output = list_of_external(args.repo_names == 1)
    if args.log:
        logger.info(
            "{:,} external {}(s): {}".format(
                len(output),
                "repo" if args.repo_names else "plugin",
                delim.join(output),
            )
        )
    else:
        print(delim.join(output))
    success = True
else:
    success = None

sys_exit(logger, NAME, args.task, success, log=args.log)



KEYWORDS = {
    "!": "run",
    "?": "help",
    "--help": "help",
    "H": "hardware",
    "I": "install",
    "K": "Kanata",
    "P": "plugins",
    "T": "terraform",
    "V": "verbose",
    "abcli": "awesome-bash-cli",
    "am": "add_month",
    "at": "add_to",
    "bp": "blue_plugin",
    "bsbc": "blue_sbc",
    "bld": "build",
    "br": "browser",
    "bs": "blue-stability",
    "bw": "blue-wiki",
    "bwiki": "blue-wiki",
    "cp": "copy",
    "det": "detect",
    "dl": "download",
    "ds": "dataset",
    "em": "earthdaily_mosaic",
    "eval": "evaluate",
    "fm": "fashion_mnist",
    "gmp": "get_model_path",
    "hf": "huggingface",
    "ic": "image_classifier",
    "id": "identify",
    "ing": "ingest",
    "iranwatch": "iran-watching",
    "Kw": "Kanata_web",
    "le": "list_of_external",
    "ls": "list",
    "mcsai": "mayo_clinic_strip_ai",
    "nbs": "notebooks-and-scripts",
    "obj": "object",
    "op": "open",
    "pk": "pack_keyword",
    "pr": "predict",
    "pub": "publish",
    "roof": "roofAI",
    "rndr": "render",
    "rv22": "RAW_Vancouver_PORTAL_2022",
    "sel": "select",
    "sl": "solid",
    "tf": "training_framework",
    "tr": "train",
    "uf": "update_fork",
    "uk": "unpack_keyword",
    "ul": "upload",
    "up": "upload",
    "upd": "update",
    "val": "validate",
    "vanwatch": "vancouver-watching",
    "yolo": "yolov5",
    "yt": "youtube",
}
